@@ -32,3 +32,11 @@ class InvalidEntityTypeError(DBDaoraError):
 
 class RequiredClassAttributeError(DBDaoraError):
     ...
+
+
+class InvalidGeoSpatialDataError(DBDaoraError):
+    ...
+
+
+class CacheNotAvailableError(DBDaoraError):
+    ...
