@@ -1,6 +1,14 @@
 from dataclasses import dataclass
 from logging import Logger, getLogger
-from typing import Any, Generic, List, Optional, Sequence, Union
+from typing import (
+    Any,
+    AsyncGenerator,
+    Generic,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 from cachetools import Cache
 
@@ -16,7 +24,7 @@ from ..repository import MemoryRepository
 class Service(Generic[Entity, EntityData, FallbackKey]):
     repository: MemoryRepository[Entity, EntityData, FallbackKey]
     circuit_breaker: AsyncCircuitBreaker
-    fallback_circuit_breaker: Optional[AsyncCircuitBreaker]
+    fallback_circuit_breaker: AsyncCircuitBreaker
     cache: Optional[Cache]
     logger: Logger
 
@@ -24,7 +32,7 @@ class Service(Generic[Entity, EntityData, FallbackKey]):
         self,
         repository: MemoryRepository[Entity, EntityData, FallbackKey],
         circuit_breaker: AsyncCircuitBreaker,
-        fallback_circuit_breaker: Optional[AsyncCircuitBreaker] = None,
+        fallback_circuit_breaker: AsyncCircuitBreaker,
         cache: Optional[Cache] = None,
         exists_cache: Optional[Cache] = None,
         logger: Logger = getLogger(__name__),
@@ -35,73 +43,34 @@ class Service(Generic[Entity, EntityData, FallbackKey]):
         self.cache = cache
         self.exists_cache = exists_cache
         self.entity_circuit = self.circuit_breaker(self.repository.entity)
-        self.entities_circuit = self.circuit_breaker(self.repository.entities)
         self.add_circuit = self.circuit_breaker(self.repository.add)
         self.delete_circuit = self.circuit_breaker(self.repository.delete)
         self.exists_circuit = self.circuit_breaker(self.repository.exists)
         self.logger = logger
 
-        if self.fallback_circuit_breaker:
-            self.entity_fallback_circuit = self.fallback_circuit_breaker(
-                self.repository.entity
-            )
-            self.entities_fallback_circuit = self.fallback_circuit_breaker(
-                self.repository.entities
-            )
-            self.add_fallback_circuit = self.fallback_circuit_breaker(
-                self.repository.add
-            )
-            self.delete_fallback_circuit = self.fallback_circuit_breaker(
-                self.repository.delete
-            )
-            self.exists_fallback_circuit = self.fallback_circuit_breaker(
-                self.repository.exists
-            )
-        else:
-            self.entity_fallback_circuit = self.repository.entity
-            self.entities_fallback_circuit = self.repository.entities
-            self.add_fallback_circuit = self.repository.add
-            self.delete_fallback_circuit = self.repository.delete
-            self.exists_fallback_circuit = self.repository.exists
+        self.entity_fallback_circuit = self.fallback_circuit_breaker(
+            self.repository.entity
+        )
+        self.add_fallback_circuit = self.fallback_circuit_breaker(
+            self.repository.add
+        )
+        self.delete_fallback_circuit = self.fallback_circuit_breaker(
+            self.repository.delete
+        )
+        self.exists_fallback_circuit = self.fallback_circuit_breaker(
+            self.repository.exists
+        )
 
-    async def get_many(self, *ids: str, **filters: Any) -> Sequence[Any]:
-        try:
-            if self.cache is None:
-                return [
-                    entity
-                    for entity in await self.entities_circuit(
-                        self.repository.query(many=ids, **filters)
-                    )
-                    if entity is not None
-                ]
+    def get_many(
+        self, *ids: str, memory: bool = True, **filters: Any
+    ) -> AsyncGenerator[Entity, None]:
+        if self.cache is None:
+            if memory:
+                return self.get_many_no_cache_memory(*ids, **filters)
 
-            return await self.get_many_cached(ids, self.cache, **filters)
+            return self.get_many_no_cache_fallback(*ids, **filters)
 
-        except DBDaoraCircuitBreakerError as error:
-            self.logger.warning(error)
-            if self.should_raise_not_found_error_for_fallback_circuit_breaker(
-                error
-            ):
-                raise EntityNotFoundError(ids, filters)
-
-            try:
-                if self.cache is not None:
-                    return await self.get_many_cached(
-                        ids, self.cache, memory=False, **filters
-                    )
-
-                return [
-                    entity
-                    for entity in await self.entities_fallback_circuit(
-                        self.repository.query(
-                            many=ids, memory=False, **filters
-                        )
-                    )
-                    if entity is not None
-                ]
-            except DBDaoraCircuitBreakerError as fallback_error:
-                self.logger.warning(fallback_error)
-                raise EntityNotFoundError(ids, filters)
+        return self.get_many_cached(ids, self.cache, **filters)
 
     def should_raise_not_found_error_for_fallback_circuit_breaker(
         self, error: DBDaoraCircuitBreakerError
@@ -118,82 +87,179 @@ class Service(Generic[Entity, EntityData, FallbackKey]):
 
         return False
 
+    async def get_many_no_cache_memory(
+        self, *ids: str, **filters: Any
+    ) -> AsyncGenerator[Entity, None]:
+        try:
+            try:
+                async for entity in self.repository.query(
+                    many=ids, **filters
+                ).entities:
+                    yield entity
+
+                self.circuit_breaker.set_success()
+
+            except self.circuit_breaker.expected_exception as error:
+                self.circuit_breaker.set_failure('get-many-memory', error)
+                raise
+
+        except DBDaoraCircuitBreakerError as error:
+            self.logger.warning(error)
+            if self.should_raise_not_found_error_for_fallback_circuit_breaker(
+                error
+            ):
+                raise EntityNotFoundError(ids, filters)
+
+            try:
+                try:
+                    async for entity in self.repository.query(
+                        many=ids, memory=False, **filters
+                    ).entities:
+                        yield entity
+
+                    self.fallback_circuit_breaker.set_success()
+
+                except self.fallback_circuit_breaker.expected_exception as error:
+                    self.fallback_circuit_breaker.set_failure(
+                        'get-many-fallback', error
+                    )
+                    raise
+
+            except DBDaoraCircuitBreakerError as fallback_error:
+                self.logger.warning(fallback_error)
+                raise EntityNotFoundError(ids, filters)
+
+    async def get_many_no_cache_fallback(
+        self, *ids: str, **filters: Any
+    ) -> AsyncGenerator[Entity, None]:
+        try:
+            try:
+                async for entity in self.repository.query(
+                    many=ids, memory=False, **filters
+                ).entities:
+                    yield entity
+
+                self.fallback_circuit_breaker.set_success()
+
+            except self.fallback_circuit_breaker.expected_exception as error:
+                self.fallback_circuit_breaker.set_failure(
+                    'get-many-fallback', error
+                )
+                raise
+
+        except DBDaoraCircuitBreakerError as fallback_error:
+            self.logger.warning(fallback_error)
+            raise EntityNotFoundError(ids, filters)
+
     async def get_many_cached(
         self,
         ids: Sequence[str],
         cache: Cache,
         memory: bool = True,
         **filters: Any,
-    ) -> Sequence[Any]:
-        missed_ids: List[str] = []
-        entities = {}
+    ) -> AsyncGenerator[Entity, None]:
         cache_key_suffix = self.cache_key_suffix(**filters)
+        is_composed_key = False
 
         for id_ in ids:
             entity = self.get_cached_entity(id_, cache_key_suffix, **filters)
 
             if entity is None:
-                missed_ids.append(id_)
+                if not is_composed_key and isinstance(id_, tuple):
+                    is_composed_key = True
 
-            entities[id_] = entity
-
-        if missed_ids:
-            try:
-                if memory:
-                    missed_entities = await self.entities_circuit(
-                        self.repository.query(many=missed_ids, **filters)
+                try:
+                    entity = await self.repository_entity(
+                        id_, cache_key_suffix, memory, **filters
                     )
-                else:
-                    missed_entities = await self.entities_fallback_circuit(
-                        self.repository.query(
-                            many=missed_ids, memory=False, **filters
+                    if entity:
+                        yield entity
+                        self.set_cached_entity(id_, cache_key_suffix, entity)
+
+                except DBDaoraCircuitBreakerError as error:
+                    self.logger.warning(error)
+                    if self.should_raise_not_found_error_for_fallback_circuit_breaker(
+                        error
+                    ):
+                        continue
+
+                    try:
+                        entity = await self.repository_entity(
+                            id_, cache_key_suffix, memory=False, **filters
                         )
-                    )
+                        if entity:
+                            yield entity
+                            self.set_cached_entity(
+                                id_, cache_key_suffix, entity
+                            )
 
-            except EntityNotFoundError:
-                missed_entities = []
+                    except DBDaoraCircuitBreakerError as error:
+                        self.logger.warning(error)
 
-            for entity in missed_entities:
-                id_ = (
-                    entity[self.repository.id_name]
-                    if isinstance(entity, dict)
-                    else entity
-                    if isinstance(entity, str)
-                    else getattr(entity, self.repository.id_name)
-                )
-                entities[id_] = entity
+            elif entity is not CACHE_ALREADY_NOT_FOUND:
+                yield entity
 
-        final_entities = []
+    async def repository_entity(
+        self,
+        id_: Union[str, Tuple[str, ...]],
+        cache_key_suffix: str,
+        memory: bool,
+        **filters: Any,
+    ) -> Optional[Entity]:
+        circuit = (
+            self.entity_circuit if memory else self.entity_fallback_circuit
+        )
 
-        for id_, entity in entities.items():
-            if entity is not None and entity is not CACHE_ALREADY_NOT_FOUND:
-                final_entities.append(entity)
-                self.set_cached_entity(id_, cache_key_suffix, entity)
-
-            elif entity is None:
-                self.set_cached_entity(
-                    id_, cache_key_suffix, CACHE_ALREADY_NOT_FOUND
+        try:
+            if isinstance(id_, tuple):
+                return await circuit(
+                    self.repository.query(*id_, memory=memory, **filters)
                 )
 
-        if not final_entities:
-            raise EntityNotFoundError(ids, filters)
+            return await circuit(
+                self.repository.query(id_, memory=memory, **filters)
+            )
+        except EntityNotFoundError:
+            self.set_cached_entity(
+                id_, cache_key_suffix, CACHE_ALREADY_NOT_FOUND,
+            )
+            return None
 
-        return final_entities
+    def entity_id(
+        self, entity: Entity, is_composed_key: bool
+    ) -> Union[str, Tuple[str, ...]]:
+        if is_composed_key:
+            return tuple(
+                entity[id_name]
+                if isinstance(entity, dict)
+                else entity
+                if isinstance(entity, str)
+                else getattr(entity, id_name)
+                for id_name in self.repository.many_key_attrs
+            )
+
+        return (
+            entity[self.repository.id_name]
+            if isinstance(entity, dict)
+            else entity
+            if isinstance(entity, str)
+            else getattr(entity, self.repository.id_name)
+        )
 
     def get_cached_entity(
-        self, id: str, key_suffix: str, **filters: Any,
+        self, id: Union[str, Tuple[str, ...]], key_suffix: str, **filters: Any,
     ) -> Any:
         if self.cache is None:
             return None
 
         return self.cache.get(self.cache_key(id, key_suffix))
 
-    def cache_key(self, id: str, suffix: str) -> str:
+    def cache_key(self, id: Union[str, Tuple[str, ...]], suffix: str) -> str:
         return f'{id}{suffix}'
 
     def set_cached_entity(
         self,
-        id: str,
+        id: Union[str, Tuple[str, ...]],
         key_suffix: str,
         entity: Union[Entity, 'CacheAlreadyNotFound'],
     ) -> None:
@@ -207,7 +273,7 @@ class Service(Generic[Entity, EntityData, FallbackKey]):
 
     async def get_one(self, id: Optional[str] = None, **filters: Any) -> Any:
         if id is not None:
-            filters['id'] = id
+            filters[self.repository.id_name] = id
 
         try:
             if self.cache is None:
@@ -225,6 +291,8 @@ class Service(Generic[Entity, EntityData, FallbackKey]):
                 raise EntityNotFoundError(id, filters)
 
             try:
+                filters.pop('memory', None)
+
                 if self.cache is not None:
                     return await self.get_one_cached(
                         cache=self.cache, memory=False, **filters
@@ -280,7 +348,11 @@ class Service(Generic[Entity, EntityData, FallbackKey]):
         return entity
 
     async def add(
-        self, entity: Any, *entities: Any, memory: bool = True
+        self,
+        entity: Any,
+        *entities: Any,
+        memory: bool = True,
+        fallback_ttl: Optional[int] = None,
     ) -> None:
         if not memory:
             try:
@@ -316,7 +388,7 @@ class Service(Generic[Entity, EntityData, FallbackKey]):
         self, entity_id: Optional[str] = None, **filters: Any
     ) -> None:
         if entity_id is not None:
-            filters['id'] = entity_id
+            filters[self.repository.id_name] = entity_id
 
         try:
             await self.delete_circuit(self.repository.query(**filters))
